//---------------------------------------------------------------------------
// Files.cpp
//
// Version: 1.2.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <mbstring.h>
#include "Files.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Moochos { namespace Moa { namespace Files {
//---------------------------------------------------------------------------
int __fastcall TFileNameList::CompareStrings(const AnsiString S1, const AnsiString S2)
{
	return AnsiCompareFileName(S1, S2);
}
//---------------------------------------------------------------------------
__fastcall TFileNameList::TFileNameList() :
	TStringList()
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TFileNameList::~TFileNameList()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TFileInfoEnumerator::FindFirst(const AnsiString& AFileName)
{
	HANDLE newHandle = ::FindFirstFile(AFileName.c_str(), &FData);
	if(newHandle == INVALID_HANDLE_VALUE) {
		FFound = false;
		return;
	}
	FHandle = newHandle;
	FFound = true;
}
//---------------------------------------------------------------------------
void __fastcall TFileInfoEnumerator::FindNext()
{
	if(!::FindNextFile(FHandle, &FData)) {
		FindClose();
		FFound = false;
		return;
	}
}
//---------------------------------------------------------------------------
void __fastcall TFileInfoEnumerator::FindClose()
{
	if(FHandle == NULL) {
		return;
	}
	::FindClose(FHandle);
	FHandle = NULL;
}
//---------------------------------------------------------------------------
__fastcall TFileInfoEnumerator::TFileInfoEnumerator(const AnsiString& AFileName) :
	FHandle(NULL),
	FData(),
	FFound(false)
{
	FindFirst(AFileName);
}
//---------------------------------------------------------------------------
__fastcall TFileInfoEnumerator::~TFileInfoEnumerator()
{
	FindClose();
}
//---------------------------------------------------------------------------
bool __fastcall TFileInfoEnumerator::HasNext()
{
	return FFound;
}
//---------------------------------------------------------------------------
WIN32_FIND_DATA __fastcall TFileInfoEnumerator::GetNext()
{
	WIN32_FIND_DATA next;
	GetNext(next);
	return next;
}
//---------------------------------------------------------------------------
void __fastcall __fastcall TFileInfoEnumerator::GetNext(WIN32_FIND_DATA& ANext)
{
	if(!HasNext()) {
		throw EFileError("The program cannot find any more files.");
	}
	ANext = FData;
	FindNext();
}
//---------------------------------------------------------------------------
AnsiString __fastcall ExtractLongPathName(const AnsiString AFileName)
{
	if(AFileName.Length() >= MAX_PATH) {
		return AnsiString();
	}
	const DWORD bufferSize = MAX_PATH;
	TCHAR buffer[bufferSize] = {0};
	DWORD didCopySize = ::GetLongPathName(AFileName.c_str(), buffer, bufferSize);
	if(didCopySize == 0 || didCopySize > bufferSize) {
		return AnsiString();
	}
	return AnsiString(buffer, didCopySize);
}
//---------------------------------------------------------------------------
bool __fastcall DriveExists(const AnsiString& AFileName)
{
	return GetDriveType(AFileName.c_str()) != 1;
}
//---------------------------------------------------------------------------
bool __fastcall FileOrDirExists(const AnsiString& AFileName)
{
	return GetFileAttributes(AFileName.c_str()) != 0xFFFFFFFF;
}
//---------------------------------------------------------------------------
bool __fastcall IsFileNameMask(const AnsiString& AFileName)
{
	return _mbspbrk(AFileName.c_str(), "*?") != NULL;
}
//---------------------------------------------------------------------------
bool __fastcall IsAbsoluteFileName(const AnsiString& AFileName)
{
	return _mbschr(AFileName.c_str(), ':') != NULL;
}
//---------------------------------------------------------------------------
bool __fastcall IsRelativeFileName(const AnsiString& AFileName)
{
	return !IsAbsoluteFileName(AFileName);
}
//---------------------------------------------------------------------------
bool __fastcall IsDirectory(const WIN32_FIND_DATA& AFindData)
{
	return (AFindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) != 0;
}
//---------------------------------------------------------------------------
bool __fastcall IsDotFileName(const WIN32_FIND_DATA& AFindData)
{
	return lstrcmpi(AFindData.cFileName, ".") == 0;
}
//---------------------------------------------------------------------------
bool __fastcall IsDotDotFileName(const WIN32_FIND_DATA& AFindData)
{
	return lstrcmpi(AFindData.cFileName, "..") == 0;
}
//---------------------------------------------------------------------------
__fastcall TFileTimes::TFileTimes() :
	Items()
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TFileTimes::TFileTimes(const TFileTimes& ABase, const TFileTimeKinds& AContextKinds, const TFileTimes& AContext) :
	Items()
{
	for(int i = 0; i < Count; i++) {
		Items[i] = (AContextKinds.Contains(static_cast<TFileTimeKind>(i)) ? AContext : ABase).Items[i];
	}
}
//---------------------------------------------------------------------------
__fastcall TFileTimes::TFileTimes(const FILETIME& AFileTime) :
	Items()
{
	for(int i = 0; i < Count; i++) {
		Items[i] = AFileTime;
	}
}
//---------------------------------------------------------------------------
__fastcall TFileTimes::TFileTimes(const FILETIME& ACreationTime, const FILETIME& ALastWriteTime, const FILETIME& ALastAccessTime) :
	Items()
{
	Items[ftCreationTime] = ACreationTime;
	Items[ftLastWriteTime] = ALastWriteTime;
	Items[ftLastAccessTime] = ALastAccessTime;
}
//---------------------------------------------------------------------------
bool __fastcall GetFileTimes(const AnsiString& AFileName, const TFileTimeKinds& AFileTimeKinds, TFileTimes& AFileTimes)
{
	WIN32_FIND_DATA fileInfo;
	{
		TFileInfoEnumerator fileInfoEnumerator(AFileName);
		if(!fileInfoEnumerator.HasNext()) {
			return false;
		}
		fileInfo = fileInfoEnumerator.GetNext();
		if(fileInfoEnumerator.HasNext()) {
			return false;
		}
	}
	AFileTimes = TFileTimes(
		AFileTimes,
		AFileTimeKinds,
		TFileTimes(fileInfo.ftCreationTime, fileInfo.ftLastWriteTime, fileInfo.ftLastAccessTime));
	return true;
}
//---------------------------------------------------------------------------
bool __fastcall SetFileTimes(const AnsiString& AFileName, const TFileTimeKinds& AFileTimeKinds, const TFileTimes& AFileTimes)
{
	HANDLE fileHandle = CreateFile(
		AFileName.c_str(),
		GENERIC_WRITE,
		0,
		NULL,
		OPEN_EXISTING,
		DirectoryExists(AFileName) ?
			FILE_ATTRIBUTE_NORMAL | FILE_FLAG_BACKUP_SEMANTICS :
			FILE_ATTRIBUTE_NORMAL,
		NULL);
	if(fileHandle == INVALID_HANDLE_VALUE) {
		return false;
	}
	try {
		if(!SetFileTime(
			fileHandle,
			AFileTimeKinds.Contains(ftCreationTime) ? &AFileTimes.Items[ftCreationTime] : NULL,
			AFileTimeKinds.Contains(ftLastAccessTime) ? &AFileTimes.Items[ftLastAccessTime] : NULL,
			AFileTimeKinds.Contains(ftLastWriteTime) ? &AFileTimes.Items[ftLastWriteTime] : NULL)) {
			return false;
		}
	}
	__finally {
		CloseHandle(fileHandle);
	}
	return true;
}
//---------------------------------------------------------------------------
}}} // End of namespace Moochos::Moa::Files
//---------------------------------------------------------------------------

